# -*- coding: utf-8 -*-
"""Action script for manifest-deliver.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os

from realog import debug
import status

from island import (
    commands,
    config,
    env,
    manifest,
    multiprocess,
    tools,
)


#
# @brief Get the global description of the current action
# @return (string) the description string (fist line if reserved for the overview, all is for the specific display)
#
def help():
    return "Deliver the manifest (merge develop vertion and create a branch with the specific current tags)"


#
# @brief Add argument to the specific action
# @param[in,out] my_args (death.Arguments) Argument manager
# @param[in] section Name of the currect action
#
def add_specific_arguments(my_args, section):
    pass


# must be on the branch we choice to merge ...


#
# @brief Execute the action required.
#
# @return error value [0 .. 50] the <0 value is reserved system ==> else, what you want.
#         None : No error (return program out 0)
#         -10 : ACTION is not existing
#         -11 : ACTION execution system error
#         -12 : ACTION Wrong parameters
#
def execute(_arguments):
    argument_remote_name = ""
    for elem in _arguments:
        debug.error("pull Wrong argument: '" + elem.get_option_name() + "' '" + elem.get_arg() + "'")

    # check system is OK
    manifest.check_island_is_init()

    configuration = config.get_unique_config()

    file_source_manifest = os.path.join(env.get_island_path_manifest(), configuration.get_manifest_name())
    if os.path.exists(file_source_manifest) is False:
        debug.error("Missing manifest file : '" + str(file_source_manifest) + "'")

    elem = configuration.get_manifest_config()

    mani = manifest.Manifest(file_source_manifest)

    destination_branch = mani.deliver_master
    source_branch = mani.deliver_develop

    # Check the manifest is up to date ...
    base_display = tools.get_list_base_display(0, 0, elem)
    debug.verbose("deliver-ckeck: " + base_display)
    if (
        status.deliver_check(
            elem,
            argument_remote_name,
            0,
            base_display,
            source_branch,
            destination_branch,
        )
        is False
    ):
        debug.error("Can not deliver a MANIFEST that is not ready to merge", crash=False)
        return env.ret_action_fail

    all_tags = check_all_tags(mani)
    if all_tags == None:
        debug.error("Need the Tags are set in sub-repository", crash=False)
        return env.ret_action_fail

    # deliver the manifest (if Needed ...)
    base_display = tools.get_list_base_display(0, 0, elem)

    debug.info("manifest-deliver: ========================================================================")
    debug.info("manifest-deliver: == " + base_display)
    debug.info("manifest-deliver: ========================================================================")

    git_repo_path = os.path.join(env.get_island_root_path(), elem.path)
    # Check the validity of the version,
    version_description, add_in_version_management = status.get_current_version_repo(git_repo_path)
    if version_description == None:
        return env.ret_action_fail
    debug.info("manifest-deliver:     ==> version: " + str(version_description))

    # go to the dev branch
    select_branch = commands.get_current_branch(git_repo_path)

    # create new repo tag
    new_version_description = status.create_new_version_repo(
        git_repo_path,
        version_description,
        add_in_version_management,
        source_branch,
        destination_branch,
    )
    debug.info("new version: " + str(new_version_description))
    if new_version_description == None:
        return
    # merge branch
    commands.checkout(git_repo_path, destination_branch)
    if mani.deliver_mode == "merge":
        merge_force = True
    else:
        merge_force = False
    commands.merge_branch_on_master(git_repo_path, source_branch, merge_force, branch_destination=destination_branch)

    manifest.tag_manifest(file_source_manifest, all_tags)

    version_path_file = os.path.join(git_repo_path, "version.txt")
    # update version file:
    tools.file_write_data(version_path_file, tools.version_to_string(new_version_description))
    commands.add_file(git_repo_path, version_path_file)
    commands.commit_all(
        git_repo_path,
        "[RELEASE] Release v" + tools.version_to_string(new_version_description),
    )
    commands.tag(git_repo_path, "v" + tools.version_to_string(new_version_description))
    commands.checkout(git_repo_path, source_branch)
    commands.reset_hard(git_repo_path, destination_branch)
    new_version_description.append("dev")
    manifest.tag_clear(file_source_manifest)
    tools.file_write_data(version_path_file, tools.version_to_string(new_version_description))
    commands.add_file(git_repo_path, version_path_file)
    commands.commit_all(git_repo_path, status.default_update_message)
    commands.checkout(git_repo_path, destination_branch)

    debug.info("manifest-deliver: ==> DONE")


def check_all_tags(mani):
    all_project = mani.get_all_configs()
    debug.info("Check all: " + str(len(all_project)) + " projects have a current tag ...")
    id_element = 0
    check_have_error = False
    list_tags = []
    for elem in all_project:
        id_element += 1
        base_display = tools.get_list_base_display(id_element, len(all_project), elem)
        if elem.volatile is True:
            debug.info(base_display + "\r\t\t\t\t\t\t\t\t\t" + "     (Not Managed)")
            continue
        tags_comment = ""
        git_repo_path = os.path.join(env.get_island_root_path(), elem.path)
        if os.path.exists(git_repo_path) is False:
            debug.error(
                base_display + volatile + "\r\t\t\t\t\t\t\t\t\t" + "     (not download)",
                crash=False,
            )
            check_have_error = True
            continue
        ret_current_tags = commands.get_tags_current(git_repo_path)
        debug.verbose("tags found: " + str(ret_current_tags))
        if len(ret_current_tags) == 0:
            list_tags.append({"name": elem.name, "tag": ""})
        else:
            list_tags.append({"name": elem.name, "tag": ret_current_tags[0]})
        for elem_tag in ret_current_tags:
            if len(tags_comment) != 0:
                tags_comment += ","
            tags_comment += elem_tag
        if len(ret_current_tags) == 0:
            debug.error(
                base_display + "\r\t\t\t\t\t\t\t\t\t" + "     (NO TAG DETECTED)",
                crash=False,
            )
            check_have_error = True
            continue
        else:
            debug.info(base_display + "\r\t\t\t\t\t\t\t\t\t" + "     " + tags_comment)
    if check_have_error is True:
        return None
    return list_tags



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""System call Interface.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import shlex
import subprocess
import sys

# Local import
from realog import debug


def generic_display_error(
    return_value,
    type_name,
    error_only=False,
    available_return=[0],
    display_if_nothing=True,
):
    debug.verbose(str(return_value))
    if return_value[0] in available_return:
        if error_only is True:
            return
        display = False
        if return_value[1] != "":
            debug.info(return_value[1])
            display = True
        if return_value[2] != "":
            debug.warning(return_value[2])
            display = True
        if display_if_nothing is False:
            return
        if display is False:
            debug.verbose("GIT(" + type_name + "): All done OK")
    else:
        display = False
        if return_value[1] != "":
            debug.warning("ERROR GIT(" + type_name + ") 1:" + return_value[1])
            display = True
        if return_value[2] != "":
            debug.warning("ERROR GIT(" + type_name + ") 2:" + return_value[2])
            display = True
        if display is False:
            debug.warning("ERROR GIT(" + type_name + "): Unknow error return_value=" + str(return_value[0]))


def run_command_direct_shell(cmd_line, cwd=None, shell=False):
    # prepare command line:
    args = shlex.split(cmd_line)
    debug.verbose("cmd = " + str(args))
    subprocess.check_call(args, shell=shell)
    return ""


#
# @brief Execute the command and ruturn generate data
#
def run_command_direct(cmd_line, cwd=None):
    # prepare command line:
    args = shlex.split(cmd_line)
    debug.verbose("cmd = " + str(args))
    """
	if True:
		subprocess.check_call(args)
		return ""
	"""
    try:
        # create the subprocess
        # p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        # p = subprocess.check_call(args)
        """
        if cwd != None:
                debug.info("path = " + cwd)
        """
        p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=cwd)
    except subprocess.CalledProcessError as e:
        debug.error("subprocess.CalledProcessError : " + str(args))
    except:
        debug.error("Exception on : " + str(args))
    # launch the subprocess:
    output, err = p.communicate()
    if sys.version_info >= (3, 0):
        output = output.decode("utf-8")
        err = err.decode("utf-8")
    # Check errors:
    if p.returncode == 0:
        if output == None:
            return err[:-1]
        return output[:-1]
    else:
        return False


def run_command(cmd_line, cwd=None):
    # prepare command line:
    args = shlex.split(cmd_line)
    debug.verbose("cmd = " + str(args))
    try:
        # create the subprocess
        """
        if cwd != None:
                debug.info("path = " + cwd)
        """
        p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=cwd)
    except subprocess.CalledProcessError as ex:
        debug.error(f"subprocess.CalledProcessError : {args} ==> {ex}")
    except Exception as ex:
        debug.error(f"Exception : {args} ==> {ex}")
    # launch the subprocess:
    output, err = p.communicate()
    if sys.version_info >= (3, 0):
        output = output.decode("utf-8")
        err = err.decode("utf-8")
    # Check error :
    return [p.returncode, output[:-1], err[:-1]]



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Toolbox.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import fnmatch
import os
import shutil
import time

# Local import
from realog import debug

from . import env


def get_run_path() -> str:
    return os.getcwd()


def get_current_path(file) -> str:
    return os.path.dirname(os.path.realpath(file))


def create_directory(path):
    try:
        os.stat(path)
    except:
        os.makedirs(path)


def create_directory_of_file(file):
    path = os.path.dirname(file)
    create_directory(path)


def get_list_sub_path(path):
    # TODO : os.listdir(path)
    for dirname, dirnames, filenames in os.walk(path):
        return dirnames
    return []


def get_list_sub_files(path):
    # TODO : os.listdir(path)
    for dirname, dirnames, filenames in os.walk(path):
        return filenames
    return []


def remove_path_and_sub_path(path):
    if os.path.isdir(path):
        debug.verbose("remove path : '" + path + "'")
        shutil.rmtree(path)


def remove_file(path):
    if os.path.isfile(path):
        os.remove(path)
    elif os.path.islink(path):
        os.remove(path)


def file_size(path):
    if not os.path.isfile(path):
        return 0
    statinfo = os.stat(path)
    return statinfo.st_size


def file_read_data(path, binary=False):
    if not os.path.isfile(path):
        return ""
    if binary is True:
        file = open(path, "rb")
    else:
        file = open(path, "r")
    data_file = file.read()
    file.close()
    return data_file


def version_to_string(version):
    version_ID = ""
    for id in version:
        if len(version_ID) != 0:
            if type(id) == str:
                version_ID += "-"
            else:
                version_ID += "."
        version_ID += str(id)
    return version_ID


def version_string_to_list(version):
    debug.verbose("parse version string '" + version + "'")
    out = []
    if version == "":
        return [0, 0, 0]
    elems = version.split("-")
    if len(elems[0].split(".")) <= 1:
        debug.error("Can not parde a version with wrong version model '" + version + "'")
    for elem in elems[0].split("."):
        out.append(int(elem))
    if len(elems) >= 2:
        out.append(elems[1])
    return out


#
# @brief Write data in a specific path.
# @param[in] path Path of the data might be written.
# @param[in] data Data To write in the file.
# @param[in] only_if_new (default: False) Write data only if data is different.
# @return True Something has been copied
# @return False Nothing has been copied
#
def file_write_data(path, data: str, only_if_new: bool = False):
    if only_if_new is True:
        if os.path.exists(path) is True:
            old_data = file_read_data(path)
            if old_data == data:
                return False
    # real write of data:
    create_directory_of_file(path)
    with open(path, "w") as file:
        file.write(data)
    return True


def list_to_str(list):
    if isinstance(list, str):
        return list + " "
    else:
        result = ""
        # mulyiple imput in the list ...
        for elem in list:
            result += list_to_str(elem)
        return result


def add_prefix(prefix, list):
    if list is None:
        return ""
    if isinstance(list, str):
        return prefix + list
    else:
        if len(list) == 0:
            return ""
        else:
            result = []
            for elem in list:
                result.append(prefix + elem)
            return result


def store_command(cmd_line, file):
    # write cmd line only after to prevent errors ...
    if file == "" or file is None:
        return
    debug.verbose("create cmd file: " + file)
    # Create directory:
    create_directory_of_file(file)
    # Store the command Line:
    file2 = open(file, "w")
    file2.write(cmd_line)
    file2.flush()
    file2.close()


def get_type_string(in_type):
    if type(in_type) == str:
        return "string"
    elif type(in_type) == list:
        return "list"
    elif type(in_type) == dict:
        return "dict"
    return "unknown"


# List tools:
def list_append_and_check(listout, newElement, order):
    for element in listout:
        if element == newElement:
            return
    listout.append(newElement)
    if order is True and not isinstance(newElement, dict):
        listout.sort()


def list_append_to(out_list, in_list, order=False):
    if type(in_list) == str:
        list_append_and_check(out_list, in_list, order)
    elif type(in_list) == list:
        # multiple input in the list ...
        for elem in in_list:
            list_append_and_check(out_list, elem, order)
    elif type(in_list) == dict:
        list_append_and_check(out_list, in_list, order)
    else:
        debug.warning(f"can not add in list other than {list/dict/str} : {type(in_list)}")


def list_append_to_2(listout, module, in_list, order=False):
    # sepcial cse of bool
    if type(in_list) == bool:
        listout[module] = in_list
        return
    # add list in the Map
    if module not in listout:
        listout[module] = []
    # add elements...
    list_append_to(listout[module], in_list, order)


#
# @brief The version number can be set in an external file to permit to have a singe position to change when create a vew version
# @param[in] path_module (string) Path of the module position
# @param[in] filename_or_version (string or list) Path of the version or the real version lint parameter
# @return (list) List of version number
#
def get_version_from_file_or_direct(path_module, filename_or_version):
    # check case of iser set the version directly
    if type(filename_or_version) == list:
        return filename_or_version
    # this use a version file
    file_data = file_read_data(os.path.join(path_module, filename_or_version))
    if len(file_data) == 0:
        debug.warning("not enough data in the file version size=0 " + path_module + " / " + filename_or_version)
        return [0, 0, 0]
    lines = file_data.split("\n")
    if len(lines) != 1:
        debug.warning(
            f"More than one line in the file version ==> bas case use mode: 'XX', XX.YYY', 'XX.Y.ZZZ' or 'XX.Y-dev' : {path_module} / {filename_or_version}"
        )
        return [0, 0, 0]
    line = lines[0]
    debug.debug(f"Parse line: '{line}'")
    # check if we have "-dev"
    dev_mode = ""
    list_tiret = line.split("-")
    if len(list_tiret) > 2:
        debug.warning(f"more than one '-' in version file {filename_or_version} : '{list_tiret}' in '{path_module}'")
    if len(list_tiret) >= 2:
        dev_mode = list_tiret[1]
        line = list_tiret[0]
    out = []
    list_elem = line.split(".")
    for elem in list_elem:
        out.append(int(elem))
    if dev_mode != "":
        out.append(dev_mode)
    debug.debug(f"    ==> {out}")
    return out


#
# @brief Get the list of the authors from an input list or a file
# @param[in] path_module (string) Path of the module position
# @param[in] filename_or_version (string or list) Path of the author file or the real list of authors
# @return (list) List of authors
#
def get_maintainer_from_file_or_direct(path_module, filename_or_author):
    # check case of it is set the version directly
    if isinstance(filename_or_author, list):
        return filename_or_author
    # this use a version file
    file_data = file_read_data(os.path.join(path_module, filename_or_author))
    if len(file_data) == 0:
        debug.warning(f"not enough data in the file author size=0 {path_module}/{filename_or_author}")
        return []
    # One user by line and # for comment line
    out = []
    for elem in file_data.split("\n"):
        if len(elem) == 0:
            continue
        if elem[0] == "#":
            # comment ...
            continue
        out.append(elem)
    return out


def remove_element(data, to_remove):
    base_data = []
    for elem in data:
        if type(elem) == list:
            for elem2 in elem:
                base_data.append(elem2)
        else:
            base_data.append(elem)
    base_remove = []
    for elem in to_remove:
        if type(elem) == list:
            for elem2 in elem:
                base_remove.append(elem2)
        else:
            base_remove.append(elem)
    out = []
    for elem in base_data:
        if elem not in base_remove:
            out.append(elem)
    return out


def get_list_base_display(id, count, elem):
    if env.get_display_folder_instead_of_git_name() is False:
        return f"{id}/{count} : {elem.name}"
    return f"{id}/{count} : {elem.path}"


is_first_time_sleep = True


def wait_for_server_if_needed():
    global is_first_time_sleep
    if is_first_time_sleep is False:
        is_first_time_sleep = True
        return
    if env.get_wait_between_sever_command() != 0:
        debug.info(f"Wait for server contrition ({env.get_wait_between_sever_command()} s)")
        time.sleep(env.get_wait_between_sever_command())


def filter_name_and_file(root, list_files, filter):
    # filter elements:
    tmp_list = fnmatch.filter(list_files, filter)
    out = []
    for elem in tmp_list:
        if os.path.isfile(os.path.join(root, elem)) is True:
            out.append(elem)
    return out


def filter_name(list_files, filter):
    # filter elements:
    return fnmatch.filter(list_files, filter)


def exclude_list(list_elements, filter):
    out = []
    for elem in list_elements:
        if elem not in filter:
            out.append(elem)
    return out


def import_path_local(path, limit_sub_folder=1, exclude_path=[], base_name="*"):
    out = []
    debug.debug(f"island files: {path} [START] {limit_sub_folder}")
    if limit_sub_folder == 0:
        debug.verbose("Sub-parsing limitation append ...")
        return []
    list_files = get_list_sub_files(path)
    # filter elements:
    debug.debug(f"island files: {path} : {list_files}")
    tmp_list_island_file = filter_name_and_file(path, list_files, base_name)
    debug.debug(f"island files (filtered): {path} : {tmp_list_island_file}")
    # Import the module:
    for filename in tmp_list_island_file:
        out.append(os.path.join(path, filename))
        debug.debug(f"     Find a file : '{out[-1]}'")
    list_folders_full = get_list_sub_path(path)
    list_folders = []
    for elem in list_folders_full:
        if elem in exclude_path:
            debug.verbose(f"find '{elem}' in exclude_path={exclude_path}")
            continue
        list_folders.append(os.path.join(path, elem))
    # check if we need to parse sub_folder
    if len(list_folders) != 0:
        debug.debug(f"     Find a folder : {list_folders}")
        for folder in list_folders:
            tmp_out = import_path_local(folder, limit_sub_folder - 1, exclude_path, base_name)
            # add all the elements:
            for elem in tmp_out:
                out.append(elem)
    return out



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Environment management.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os

# Local import
from realog import debug


system_base_name = "island"


def set_system_base_name(val):
    global system_base_name
    system_base_name = val
    debug.debug("Set basename: '" + str(system_base_name) + "'")


def get_system_base_name():
    global system_base_name
    return system_base_name


def get_system_config_name():
    global system_base_name
    return system_base_name + "Config.py"


fetch_manifest = True


def set_fetch_manifest(val):
    global fetch_manifest
    fetch_manifest = val


def get_fetch_manifest():
    global fetch_manifest
    return fetch_manifest


wait_between_sever_command = 0


def set_wait_between_sever_command(val):
    global wait_between_sever_command
    wait_between_sever_command = val


def get_wait_between_sever_command():
    global wait_between_sever_command
    return wait_between_sever_command


filter_command = ""


def set_filter_command(val):
    global filter_command
    filter_command = val


def get_filter_command():
    global filter_command
    return filter_command


def need_process_with_filter(data):
    global filter_command
    if filter_command == "":
        return True
    if len(data) < len(filter_command):
        return False
    if data[: len(filter_command)] == filter_command:
        return True
    return False


display_folder_instead_of_git_name = True


def set_display_folder_instead_of_git_name(val):
    global display_folder_instead_of_git_name
    display_folder_instead_of_git_name = val


def get_display_folder_instead_of_git_name():
    global display_folder_instead_of_git_name
    return display_folder_instead_of_git_name


island_root_path = os.path.join(os.getcwd())
if os.path.exists(os.path.join(island_root_path, "." + get_system_base_name())) is True:
    # all is good ...
    pass
elif os.path.exists(os.path.join(island_root_path, "..", "." + get_system_base_name())):
    island_root_path = os.path.join(os.getcwd(), "..")
elif os.path.exists(os.path.join(island_root_path, "..", "..", "." + get_system_base_name())):
    island_root_path = os.path.join(os.getcwd(), "..", "..")
elif os.path.exists(os.path.join(island_root_path, "..", "..", "..", "." + get_system_base_name())):
    island_root_path = os.path.join(os.getcwd(), "..", "..", "..")
elif os.path.exists(os.path.join(island_root_path, "..", "..", "..", "..", "." + get_system_base_name())):
    island_root_path = os.path.join(os.getcwd(), "..", "..", "..", "..")
elif os.path.exists(os.path.join(island_root_path, "..", "..", "..", "..", "..", "." + get_system_base_name())):
    island_root_path = os.path.join(os.getcwd(), "..", "..", "..", "..", "..")
elif os.path.exists(
    os.path.join(
        island_root_path,
        "..",
        "..",
        "..",
        "..",
        "..",
        "..",
        "." + get_system_base_name(),
    )
):
    island_root_path = os.path.join(os.getcwd(), "..", "..", "..", "..", "..", "..")
else:
    # debug.error("the root path of " + get_system_base_name() + " must not be upper that 6 parent path")
    pass
island_path_user_config = os.path.join(island_root_path, get_system_config_name())
island_path = os.path.join(island_root_path, "." + get_system_base_name())
island_path_config = os.path.join(island_path, "config.json")
island_path_manifest = os.path.join(island_path, "manifest")


def get_island_root_path() -> str:
    """Get the parent path where is define ".island" folder.

    :return: the parent path of the ".island"
    """
    global island_root_path
    return island_root_path


def get_island_path() -> str:
    global island_path
    return island_path


def get_island_path_config() -> str:
    global island_path_config
    return island_path_config


def get_island_path_manifest() -> str:
    global island_path_manifest
    return island_path_manifest


def get_island_path_user_config() -> str:
    global island_path_user_config
    return island_path_user_config


ret_manifest_is_not_existing = -5
ret_action_is_not_existing = -10
ret_action_executing_system_error = -11
ret_action_wrong_parameters = -12
ret_action_partial_done = -13
ret_action_fail = -14

ret_action_need_update = 15



# -*- coding: utf-8 -*-
"""Action script for commit.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os

from realog import debug

from island import (
    commands,
    config,
    env,
    manifest,
    multiprocess,
    tools,
)


#
# @brief Get the global description of the current action
# @return (string) the description string (fist line if reserved for the overview, all is for the specific display)
#
def help():
    return "Commit in all repository"


#
# @brief Add argument to the specific action
# @param[in,out] my_args (death.Arguments) Argument manager
# @param[in] section Name of the currect action
#
def add_specific_arguments(my_args, section):
    my_args.add("m", "message", haveParam=True, desc="Message to commit data")
    my_args.add("a", "all", desc="Commit all elements")
    my_args.add("", "amend", desc="Ammend data at the previous commit")


#
# @brief Execute the action required.
#
# @return error value [0 .. 50] the <0 value is reserved system ==> else, what you want.
#         None : No error (return program out 0)
#         -10 : ACTION is not existing
#         -11 : ACTION execution system error
#         -12 : ACTION Wrong parameters
#
def execute(_arguments):
    argument_message = ""
    argument_amend = ""
    argument_all = ""
    for elem in _arguments:
        if elem.get_option_name() == "message":
            debug.info("find message: '" + elem.get_arg() + "'")
            argument_message = ' --message "' + elem.get_arg() + '" '
        elif elem.get_option_name() == "all":
            argument_all = " --all "
        elif elem.get_option_name() == "amend":
            argument_amend = " --amend "
        else:
            debug.error("Wrong argument: '" + elem.get_option_name() + "' '" + elem.get_arg() + "'")

    # check system is OK
    manifest.check_island_is_init()

    configuration = config.get_unique_config()

    file_source_manifest = os.path.join(env.get_island_path_manifest(), configuration.get_manifest_name())
    if os.path.exists(file_source_manifest) is False:
        debug.error("Missing manifest file : '" + str(file_source_manifest) + "'")
    mani = manifest.Manifest(file_source_manifest)

    all_project = mani.get_all_configs()
    debug.info("commit : " + str(len(all_project)) + " projects")
    id_element = 0
    for elem in all_project:
        id_element += 1
        base_display = tools.get_list_base_display(id_element, len(all_project), elem)
        debug.info("commit: " + base_display)
        git_repo_path = os.path.join(env.get_island_root_path(), elem.path)
        if os.path.exists(git_repo_path) is False:
            debug.error("can not commit project that not exist")
            continue

        if os.path.exists(os.path.join(git_repo_path, ".git")) is False:
            # path already exist but it is not used to as a git repo ==> this is an error
            debug.warning("path '" + git_repo_path + "' is already existing but not used for a git repository. Clean it and restart")
            continue

        # simply update the repository ...
        debug.verbose("commit in project:")
        # fetch the repository
        cmd = "git commit " + argument_amend + argument_all + argument_message
        debug.debug("execute : " + cmd)
        multiprocess.run_command_direct(cmd, cwd=git_repo_path)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Inland main().

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import copy
import fnmatch
import os
import sys
from typing import Union

import death.ArgElement as arg_element
import death.Arguments as arguments
from realog import debug

# Local import
from . import (
    actions,
    env,
    host,
    tools,
)


is_init = False

debug.set_display_on_error("    ==========================\n    ==  Some error occurred  ==\n    ==========================")


def init() -> None:
    """Global initialization of Island."""
    global is_init
    if is_init is True:
        return
    # import local island files
    list_of_island_files = tools.import_path_local(
        os.path.join(tools.get_current_path(__file__), "actions"),
        base_name=env.get_system_base_name() + "*.py",
    )
    actions.init(list_of_island_files)
    # import project actions files
    list_of_island_files = tools.import_path_local(
        env.get_island_root_path(),
        2,
        [".island", ".git", "archive"],
        base_name=env.get_system_base_name() + "*.py",
    )
    actions.init(list_of_island_files)
    is_init = True


def usage(my_args) -> None:
    """Display the help of Island."""
    color = debug.get_color_set()
    # Generic argument displayed :
    my_args.display()
    print("		Action available")
    list_actions = actions.get_list_of_action()
    for elem in list_actions:
        print(f"			{color['green']}{elem}{color['default']}")
        print(f"					{actions.get_action_help(elem)}")
    """
    print("		{color['green']}init{color['default']}")
    print("			initialize a 'island' interface with a manifest in a git ")
    print("		{color['green']}sync{color['default']}")
    print("			Synchronize the current environnement")
    print("		{color['green']}status{color['default']}")
    print("			Dump the status of the environnement")
    """
    print(f"	ex: {sys.argv[0]} -c init http://github.com/atria-soft/manifest.git")
    print(f"	ex: {sys.argv[0]} sync")
    exit(0)


def check_boolean(value: Union[bool, str]) -> bool:
    """Check if the value is a real boolean or a boolean string and return the boolean value.

    :param value: Value to check.
    :return: Equivalent boolean value.
    """
    if value == "" or value == "1" or value == "true" or value == "True" or value is True:
        return True
    return False


def parse_generic_arg(my_args, argument: arg_element.ArgElement, active: bool) -> bool:
    """Keep global args that have no dependence with the mode.

    :param argument: _description_
    :param active: _description_
    :return: _description_
    """
    debug.extreme_verbose(f"parse arg : {argument.get_option_name()} {argument.get_arg()} active={active}")
    if argument.get_option_name() == "help":
        if active is False:
            usage(my_args)
        return True
    elif argument.get_option_name() == "jobs":
        if active is True:
            # multiprocess.set_core_number(int(argument.get_arg()))
            pass
        return True
    elif argument.get_option_name() == "wait":
        if active is True:
            env.set_wait_between_sever_command(int(argument.get_arg()))
        return True
    elif argument.get_option_name() == "verbose":
        if active is True:
            debug.set_level(int(argument.get_arg()))
        return True
    elif argument.get_option_name() == "folder":
        if active is True:
            env.set_display_folder_instead_of_git_name(True)
        return True
    elif argument.get_option_name() == "color":
        if active is True:
            if check_boolean(argument.get_arg()) is True:
                debug.enable_color()
            else:
                debug.disable_color()
        return True
    elif argument.get_option_name() == "filter":
        if active is True:
            env.set_filter_command(str(argument.get_arg()))
        return True
    elif argument.get_option_name() == "no-fetch-manifest":
        if active is False:
            env.set_fetch_manifest(False)
        return True
    return False


def main():
    # initialize the system ...
    init()

    debug.verbose("List of actions: " + str(actions.get_list_of_action()))

    my_args = arguments.Arguments()
    my_args.add_section("option", "Can be set one time in all case")
    my_args.add("h", "help", desc="Display this help")
    my_args.add(
        "v",
        "verbose",
        list=[
            ["0", "None"],
            ["1", "error"],
            ["2", "warning"],
            ["3", "info"],
            ["4", "debug"],
            ["5", "verbose"],
            ["6", "extreme_verbose"],
        ],
        desc="display debug level (verbose) default =2",
    )
    my_args.add("c", "color", desc="Display message in color")
    my_args.add("n", "no-fetch-manifest", haveParam=False, desc="Disable the fetch of the manifest")
    my_args.add(
        "F",
        "filter",
        haveParam=True,
        desc="Filter the action on a list of path or subpath: -f library",
    )
    my_args.add(
        "f",
        "folder",
        haveParam=False,
        desc="Display the folder instead of the git repository name",
    )
    my_args.add(
        "w",
        "wait",
        haveParam=True,
        desc="Wait between 2 access on the server (needed when the server is really slow to remove ssh connection) (default="
        + str(env.get_wait_between_sever_command())
        + ")",
    )
    my_args.set_stop_at(actions.get_list_of_action())
    local_argument = my_args.parse()

    # open configuration of island:
    config_file = env.get_island_path_user_config()
    if os.path.isfile(config_file) is True:
        sys.path.append(os.path.dirname(config_file))
        debug.debug(f"Find basic configuration file: '{config_file}'")
        # the file exist, we can open it and get the initial configuration:
        configuration_file = __import__(env.get_system_config_name()[:-3])

        if "get_exclude_path" in dir(configuration_file):
            data = configuration_file.get_exclude_path()
            debug.debug(f"get default config 'get_exclude_path' val='{data}'")
            env.set_exclude_search_path(data)

        if "get_default_color" in dir(configuration_file):
            data = configuration_file.get_default_color()
            debug.debug(f"get default config 'get_default_color' val='{data}'")
            parse_generic_arg(my_args, arg_element.ArgElement("color", str(data)), True)

        if "get_default_debug_level" in dir(configuration_file):
            data = configuration_file.get_default_debug_level()
            debug.debug(f"get default config 'get_default_debug_level' val='{data}'")
            parse_generic_arg(my_args, arg_element.ArgElement("verbose", str(data)), True)

        if "get_default_folder" in dir(configuration_file):
            data = configuration_file.get_default_folder()
            debug.debug(f"get default config 'get_default_folder' val='{data}'")
            parse_generic_arg(my_args, arg_element.ArgElement("folder", str(data)), True)

        if "get_default_wait" in dir(configuration_file):
            data = configuration_file.get_default_wait()
            debug.debug(f"get default config 'get_default_wait' val='{data}'")
            parse_generic_arg(my_args, arg_element.ArgElement("wait", str(data)), True)

        if "get_default_filter" in dir(configuration_file):
            data = configuration_file.get_default_filter()
            debug.debug(f"get default config 'get_default_filter' val='{data}'")
            parse_generic_arg(my_args, arg_element.ArgElement("filter", str(data)), True)


    # parse default unique argument:
    for argument in local_argument:
        parse_generic_arg(my_args, argument, True)

    # remove all generic arguments:
    new_argument_list = []
    for argument in local_argument:
        if parse_generic_arg(my_args, argument, False) is True:
            continue
        new_argument_list.append(argument)

    # now the first argument is: the action:
    if len(new_argument_list) == 0:
        debug.warning("--------------------------------------")
        debug.warning("Missing the action to do ...")
        debug.warning("--------------------------------------")
        usage(my_args)


    # TODO : move tin in actions ...
    list_actions = actions.get_list_of_action()

    action_to_do = new_argument_list[0].get_arg()
    new_argument_list = new_argument_list[1:]
    if action_to_do not in list_actions:
        debug.warning("--------------------------------------")
        debug.warning(f"Wrong action type : '{action_to_do}' available list: {list_actions}")
        debug.warning("--------------------------------------")
        usage(my_args)

    # todo : Remove this
    if action_to_do != "init" and os.path.exists(env.get_island_path()) is False:
        debug.error(f"Can not execute a island cmd if we have not initialize a config: '.{env.get_system_base_name()}' in upper 6 parent path")
        exit(-1)


    ret = actions.execute(action_to_do, my_args.get_last_parsed() + 1)

    exit(ret)



# -*- coding: utf-8 -*-
"""Update tools.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os

from realog import debug

from island import env, tools


# Update the links:
def update(configuration, mani, type_call):
    # TODO: do not remove link when not needed
    if len(configuration.get_links()) != 0 or len(mani.get_links()) != 0:
        debug.info(type_call + ": remove old links ...")
        for elem in configuration.get_links():
            base_path = os.path.join(env.get_island_root_path(), elem["destination"])
            debug.info(type_call + ":     link: " + str(base_path))
            if os.path.islink(base_path) is True:
                os.unlink(base_path)
            else:
                debug.error(
                    type_call + ": remove link is not authorised ==> not a link",
                    crash=False,
                )
                have_error = True
        configuration.clear_links()
        debug.info(type_call + ": add new links ...")
        for elem in mani.get_links():
            base_path = os.path.join(env.get_island_root_path(), elem["destination"])
            source_path = os.path.join(env.get_island_root_path(), elem["source"])
            debug.info(type_call + ":     link: " + str(base_path))
            if os.path.exists(base_path) is True:
                debug.error(
                    type_call + ": create link is not possible ==> path already exist",
                    crash=False,
                )
                have_error = True
            else:
                tools.create_directory_of_file(base_path)
                os.symlink(source_path, base_path)
                configuration.add_link(elem["source"], elem["destination"])
    configuration.store()



# -*- coding: utf-8 -*-
"""Action script for command.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os

from realog import debug

from island import (
    commands,
    config,
    env,
    manifest,
    multiprocess,
    tools,
)


#
# @brief Get the global description of the current action
# @return (string) the description string (fist line if reserved for the overview, all is for the specific display)
#
def help():
    return "Write the command you want to be executed in every repository"


#
# @brief Set the option argument are not able to check if the argument are correct or not
# @return (boolean) have parameter without arguments
#
def have_unknow_argument():
    return True


#
# @brief Execute the action required.
#
# @return error value [0 .. 50] the <0 value is reserved system ==> else, what you want.
#         None : No error (return program out 0)
#         -10 : ACTION is not existing
#         -11 : ACTION execution system error
#         -12 : ACTION Wrong parameters
#
def execute(_arguments):
    cmd = ""
    for elem in _arguments:
        debug.info("Get data element: " + str(elem.get_arg()))
        cmd += elem.get_arg() + " "

    # check system is OK
    manifest.check_island_is_init()

    configuration = config.get_unique_config()

    file_source_manifest = os.path.join(env.get_island_path_manifest(), configuration.get_manifest_name())
    if os.path.exists(file_source_manifest) is False:
        debug.error("Missing manifest file : '" + str(file_source_manifest) + "'")

    mani = manifest.Manifest(file_source_manifest)

    all_project = mani.get_all_configs()
    debug.info("status of: " + str(len(all_project)) + " projects")
    id_element = 0
    for elem in all_project:
        debug.info("------------------------------------------")
        id_element += 1
        base_display = tools.get_list_base_display(id_element, len(all_project), elem)
        debug.info("execute command : " + base_display)
        tools.wait_for_server_if_needed()
        # debug.debug("elem : " + str(elem))
        git_repo_path = os.path.join(env.get_island_root_path(), elem.path)
        if os.path.exists(git_repo_path) is False:
            debug.info("" + base_display + "\r\t\t\t\t\t\t\t\t\t" + "     (not download)")
            continue

        debug.verbose("execute : " + cmd)
        ret = multiprocess.run_command(cmd, cwd=git_repo_path)
        if ret[0] == 0:
            debug.info("ret=" + ret[1])
            debug.info("err=" + ret[2])
        else:
            debug.info("Execution ERROR")



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Action interface.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os
import sys
from typing import List, Any, Optional
import death.Arguments as arguments

# Local import
from realog import debug

from . import env


list_actions = []

__base_action_name = env.get_system_base_name() + "Action_"


def init(files) -> None:
    global list_actions
    debug.verbose(f"List of action for island: {len(files)}")
    for elem_path in files:
        debug.verbose("parse file : " + elem_path)
        base_name = os.path.basename(elem_path)
        if len(base_name) <= 3 + len(__base_action_name):
            # reject it, too small
            continue
        base_name = base_name[:-3]
        if base_name[: len(__base_action_name)] != __base_action_name:
            # reject it, wrong start file
            continue
        name_action = base_name[len(__base_action_name) :]
        debug.debug("    '" + os.path.basename(elem_path)[:-3] + "' file=" + elem_path)
        list_actions.append(
            {
                "name": name_action,
                "path": elem_path,
            }
        )


def get_list_of_action() -> List[str]:
    """Get the wall list of action available

    :return: the list of action name
    """
    global list_actions
    out = []
    for elem in list_actions:
        out.append(elem["name"])
    return out


def get_function_value(action_name: str, function_name: str, default_value: Optional[Any] = None) -> Any:
    """Get a description of an action.

    :param action_name: Name of the action
    :param function_name: Name of the function to call
    :param default_value: Returned value of the call if function does not exist, defaults to None
    :return: the requested value or the default_value
    """
    global list_actions
    for elem in list_actions:
        if elem["name"] == action_name:
            # finish the parsing
            sys.path.append(os.path.dirname(elem["path"]))
            the_action = __import__(__base_action_name + action_name)
            if function_name not in dir(the_action):
                return default_value
            method_to_call = getattr(the_action, function_name)
            return method_to_call()
    return default_value


def get_action_help(action_name: str) -> str:
    """Get the global help value of a module.

    :param action_name: Name of the action
    :return: The first line of description
    """
    value = get_function_value(action_name, "help", "---")
    return value.split("\n")[0]


def usage(arguments, action_name) -> None:
    # generic argument displayed for specific action:
    # print("Specific argument for the command: '" + action_name + "'" )
    value = get_function_value(action_name, "help")
    debug.info("Description:")
    debug.info("\t" + str(value))
    arguments.display(action_name)
    value = get_function_value(action_name, "help_example")
    if value is not None:
        debug.info("Example:")
        for elem in value.split("\n"):
            debug.info("\t" + elem)
    exit(0)


def execute(action_name, argument_start_id):
    global list_actions
    # TODO: Move here the check if action is available

    for elem in list_actions:
        if elem["name"] != action_name:
            continue
        debug.info("action: " + str(elem))
        # finish the parsing
        sys.path.append(os.path.dirname(elem["path"]))
        the_action = __import__(__base_action_name + action_name)
        my_under_args_parser = arguments.Arguments()
        my_under_args_parser.add("h", "help", desc="Help of this action")

        if "add_specific_arguments" in dir(the_action):
            the_action.add_specific_arguments(my_under_args_parser, elem["name"])
        have_unknow_argument = False
        if "have_unknow_argument" in dir(the_action):
            have_unknow_argument = the_action.have_unknow_argument()
        my_under_args = my_under_args_parser.parse(argument_start_id, have_unknow_argument)
        # search help if needed ==> permit to not duplicating code
        for elem in my_under_args:
            if elem.get_option_name() == "help":
                usage(my_under_args_parser, action_name)
                return 0
        # now we can execute:
        if "execute" not in dir(the_action):
            debug.error("execute is not implmented for this action ... '" + str(action_name) + "'")
            return -11
        debug.info("execute: " + action_name)
        for elem in my_under_args:
            debug.debug("    " + str(elem.get_option_name()) + "='" + str(elem.get_arg()) + "'")
        ret = the_action.execute(my_under_args)
        if ret == None:
            return 0
        if ret < 0:
            debug.info("    ==========================")
            debug.info("    ==  Some error occured  ==")
            debug.info("    ==========================")
        return ret
    debug.error("Can not do the action...")
    return -10



# -*- coding: utf-8 -*-
"""Action script for sync-local.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os

from realog import debug
import update_links

from island import (
    commands,
    config,
    env,
    manifest,
    multiprocess,
    tools,
)


#
# @brief Get the global description of the current action
# @return (string) the description string (fist line if reserved for the overview, all is for the specific display)
#
def help():
    return "Update all the branche to the trackin branch in local (no remote access)"


#
# @brief Add argument to the specific action
# @param[in,out] my_args (death.Arguments) Argument manager
# @param[in] section Name of the currect action
#
def add_specific_arguments(my_args, section):
    my_args.add(
        "r",
        "reset",
        haveParam=False,
        desc="Rebase the repository instead of 'reset --hard'",
    )


#
# @brief Execute the action required.
#
# @return error value [0 .. 50] the <0 value is reserved system ==> else, what you want.
#         None : No error (return program out 0)
#         -5  : env.ret_manifest_is_not_existing      : Manifest does not exit
#         -10 : env.ret_action_is_not_existing        : ACTION is not existing
#         -11 : env.ret_action_executing_system_error : ACTION execution system error
#         -12 : env.ret_action_wrong_parameters       : ACTION Wrong parameters
#         -13 : env.ret_action_partial_done           : ACTION partially done
#
def execute(_arguments):
    reset_instead_of_rebase = False
    for elem in _arguments:
        if elem.get_option_name() == "rebase":
            reset_instead_of_rebase = True
            debug.info("==> Request reset instead of rebase")
        else:
            debug.error(
                "SYNC Wrong argument: '" + elem.get_option_name() + "' '" + elem.get_arg() + "'",
                ret_value=env.ret_action_wrong_parameters,
            )

    # check system is OK
    manifest.check_island_is_init()

    configuration = config.get_unique_config()

    debug.info("update manifest : '" + str(env.get_island_path_manifest()) + "'")
    is_modify_manifest = commands.check_repository_is_modify(env.get_island_path_manifest())
    if is_modify_manifest is True:
        debug.warning("Manifest is modify")
    else:
        ret_track = commands.get_current_tracking_branch(env.get_island_path_manifest())
        is_forward = commands.is_forward(env.get_island_path_manifest(), ret_track)
        if is_forward is True:
            # fetch the repository
            debug.warning(
                "sync-local: Not update ==> the MANIFEST is forward the remote branch "
                + str(commands.get_forward(env.get_island_path_manifest(), ret_track))
            )
        else:
            debug.verbose("Check behind:")
            is_behind = commands.is_behind(env.get_island_path_manifest(), ret_track)
            if is_behind is False:
                # fetch the repository
                debug.info("sync-local: MANIFEST is up-to-date")
            else:
                if reset_instead_of_rebase is True:
                    debug.info("sync-local: MANIFEST Reset to " + ret_track)
                    commands.reset_hard(env.get_island_path_manifest(), ret_track)
                else:
                    debug.info("sync-local: MANIFEST Rebase to " + ret_track)
                    commands.rebase(env.get_island_path_manifest(), ret_track)

    file_source_manifest = os.path.join(env.get_island_path_manifest(), configuration.get_manifest_name())
    if os.path.exists(file_source_manifest) is False:
        debug.error(
            "Missing manifest file : '" + str(file_source_manifest) + "'",
            ret_value=env.ret_manifest_is_not_existing,
        )

    mani = manifest.Manifest(file_source_manifest)

    all_project = mani.get_all_configs()
    debug.info("synchronize : " + str(len(all_project)) + " projects")
    id_element = 0
    count_error = 0
    for elem in all_project:
        id_element += 1
        base_display = tools.get_list_base_display(id_element, len(all_project), elem)
        debug.info("----------------------------------------------------------------")
        debug.info("sync-local: " + base_display)
        # debug.debug("elem : " + str(elem))
        git_repo_path = os.path.join(env.get_island_root_path(), elem.path)
        if os.path.exists(git_repo_path) is False:
            # The Repository does not exist ==> Nothing to do...
            debug.warning("sync-local:    ==> Not download")
            count_error += 1
            continue

        if os.path.exists(os.path.join(git_repo_path, ".git")) is False:
            # path already exist but it is not used to as a git repo ==> this is an error
            debug.warning("sync-local: is already existing but not used for a git repository. Remove it and sync")
            count_error += 1
            continue

        # simply update the repository ...
        debug.verbose("Check modify:")
        is_modify = commands.check_repository_is_modify(git_repo_path)
        if is_modify is True:
            # fetch the repository
            debug.warning("sync-local: Not update ==> the repository is modified (pass through)")
            count_error += 1
            continue
        debug.verbose("Check tracking and local branch:")
        # get tracking branch
        ret_track = commands.get_current_tracking_branch(git_repo_path)
        select_branch = commands.get_current_branch(git_repo_path)
        debug.debug("sync-local: check: " + select_branch + " ==> " + ret_track)
        debug.verbose("Check forward:")
        is_forward = commands.is_forward(git_repo_path, ret_track)
        if is_forward is True:
            # fetch the repository
            debug.warning(
                "sync-local: Not update ==> the repository is forward the remote branch " + str(commands.get_forward(git_repo_path, ret_track))
            )
            count_error += 1
            continue
        debug.verbose("Check behind:")
        is_behind = commands.is_behind(git_repo_path, ret_track)
        if is_behind is False:
            # fetch the repository
            debug.info("sync-local: Nothing to do.")
            continue
        if reset_instead_of_rebase is True:
            debug.info("sync-local: Reset to " + ret_track)
            commands.reset_hard(git_repo_path, ret_track)
        else:
            debug.info("sync-local: Reset to " + ret_track)
            commands.rebase(git_repo_path, ret_track)
    if count_error != 0:
        debug.info("        ***********************************************************")
        debug.info("        **  local sync partial warning on " + str(count_error) + " repository")
        debug.info("        ***********************************************************")
        return env.ret_action_partial_done

    # Update the links:
    have_error = update_links.update(configuration, mani, "sync-local")
    if have_error is True:
        return -1
    return None



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Manifest interface.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""
import json
import copy
import os
from typing import List, Dict, Any

# import pyyaml module
import yaml
from yaml.loader import SafeLoader


from lxml import etree

# Local import
from realog import debug

from . import (
    config,
    env,
    multiprocess,
    repo_config,
)


def is_island_init():
    if not os.path.exists(env.get_island_path()):
        debug.verbose(f"Island is not init: path does not exist: '{env.get_island_path()}'")
        return False
    if not os.path.exists(env.get_island_path_config()):
        debug.verbose(f"Island is not init: config does not exist: '{env.get_island_path_config()}'")
        return False
    if not os.path.exists(env.get_island_path_manifest()):
        debug.verbose(f"Island is not init: Manifest does not exist: '{env.get_island_path_manifest()}'")
        return False
    return True


def check_island_is_init():
    # check if .XXX exist (create it if needed)
    if not is_island_init():
        debug.error(f"System not init: missing config: '{env.get_island_path()}'. Call <island init> first")
        exit(-1)


class ManifestVSC:
    def __init__(self, manifest_yaml: str) -> None:
        self.manifest_yaml = manifest_yaml
        self.projects: List[Dict] = []
        self.default = None
        self.default_base = {
            "remote": "origin",
            "revision": "master",  # todo: rename 'branch-release'
            "sync": False,
            "branch-develop": "dev",
            "default-branch": "dev",
        }
        self.remotes: List[Dict] = []
        self.deliver_master = "master"
        self.deliver_develop = "develop"
        self.deliver_mode = "merge"
        # load the manifest
        self._load()
        # check error in manifest (double path ...)
        self._check_double_path([])

    def get_links(self):
        return []

    def _load(self):
        debug.debug(f"manifest VSC: '{self.manifest_yaml}'")
        data = {}
        with open(self.manifest_yaml) as f:
            data = yaml.load(f, Loader=SafeLoader)
        if "repositories" not in data:
            debug.error(f"in '{self.manifest_yaml}' VSC manifest: missing root key: repositories ")
        for name, value in data["repositories"].items():
            if "type" in value and value["type"] != "git":
                debug.error(f"in '{self.manifest_yaml}' VSC manifest: unsupported type: '{value['type']}' for {name}")
            if "url" not in value:
                debug.error(f"in '{self.manifest_yaml}' VSC manifest: missing 'url' for {name}")
            url = value["url"]
            # TODO: Manage URL remote element ==> dynamic add !!! and manage http(s)://xxx.yyy/*
            url_split = url.split(":")
            if len(url_split) > 1:
                url = url_split[-1]
            version = None
            if "version" not in value:
                version = value["version"]
            self.projects.append(
                {
                    "name": url,
                    "path": name,
                    "tag": version,
                }
            )

    def _create_path_with_elem(self, element):
        # debug.info(f"create path : {json.dumps(element)}")
        path = element["path"]
        if path == "":
            path = element["name"]
            if len(path) >= 4 and path[-4:] == ".git":
                path = path[:-4]
        # debug.info(f"    generate path {path}")
        return path

    def _check_double_path(self, list_path=[], space=""):
        # debug.debug(f"{space}check path : '{self.manifest_yaml}'")
        for elem in self.projects:
            path = self._create_path_with_elem(elem)
            debug.debug(f"{space}    check path:'{path}'")
            if path in list_path:
                debug.error(f"Check Manifest error : double use of the path '{path}'")
            list_path.append(path)

    def get_all_configs(self, default=None, upper_remotes=[]):
        out = []
        if default is None:
            if self.default is not None:
                default = copy.deepcopy(self.default)
            else:
                default = copy.deepcopy(self.default_base)
        # debug.error(f" self.default={self.default}")
        # add all local project
        for elem in self.projects:
            debug.verbose(f"parse element {elem}")
            if env.need_process_with_filter(elem["name"]) is False:
                debug.info(f"Filter repository: {elem['name']}")
                continue
            conf = repo_config.RepoConfig()
            conf.name = elem["name"]
            conf.tag = elem["tag"]
            conf.path = self._create_path_with_elem(elem)

            # add default remote for the project (search in inherited element)
            for remote in self.remotes:
                debug.verbose(f"    Local Remote: {remote}")
                if remote["name"] == default["remote"]:
                    conf.remotes.append(remote)
            if len(conf.remotes) == 0:
                for remote in upper_remotes:
                    debug.verbose(f"    upper Remote: {remote}")
                    if remote["name"] == default["remote"]:
                        conf.remotes.append(remote)
            if len(conf.remotes) == 0:
                debug.error(
                    f"    No remote detected: {len(conf.remotes)} for {conf.name} with default remote name : {default['remote']} self remote: {self.remotes}"
                )

            # select default remote:
            conf.select_remote = None
            debug.debug(f"    remotes count: {len(conf.remotes)}")
            for remote in conf.remotes:
                debug.debug(f"    remote={remote}")
                debug.debug(f"    Check remote : {remote['name']} == {default['remote']}")
                debug.verbose(f"          remote={remote}")
                debug.verbose(f"          default={default}")
                if remote["name"] == default["remote"]:
                    conf.select_remote = copy.deepcopy(remote)
                    debug.debug(f"    copy select={conf.select_remote}")

                    # copy the submodule synchronization
                    conf.select_remote["sync"] = default["sync"]
                    break
            if conf.select_remote == None:
                debug.error(f"missing remote for project: {conf.name}")

            conf.branch = default["revision"]
            out.append(conf)
        # create a temporary variable to transmit the remote to includes
        upper_remotes_forward = copy.deepcopy(upper_remotes)
        for remote in self.remotes:
            upper_remotes_forward.append(remote)

        if False:
            debug.info("list of all repo:")
            for elem in out:
                debug.info(f"    '{elem.name}'")
                debug.info(f"        path: {elem.path}")
                debug.info(f"        remotes: {elem.remotes}")
                debug.info(f"        select_remote: {elem.select_remote}")
                debug.info(f"        branch: {elem.branch}")
        return out


class Manifest:
    def __init__(self, manifest_xml: str) -> None:
        self.manifest_xml = manifest_xml
        self.projects: List[Dict] = []
        self.default = None
        self.default_base = {
            "remote": "origin",
            "revision": "master",  # todo: rename 'branch-release'
            "sync": False,
            "branch-develop": "dev",
            "default-branch": "dev",
        }
        self.remotes: List[Dict] = []
        self.includes: List[Dict] = []
        self.imports: List[Dict] = []
        self.links: List[Dict] = []
        self.deliver_master = "master"
        self.deliver_develop = "develop"
        self.deliver_mode = "merge"
        # load the manifest
        self._load()
        # check error in manifest (double path ...)
        self._check_double_path([])

    def get_links(self) -> Dict[str, Any]:
        return self.links

    def _load(self) -> None:
        debug.debug(f"manifest : '{self.manifest_xml}'")
        tree = etree.parse(self.manifest_xml)
        root = tree.getroot()
        if root.tag != "manifest":
            debug.error(f"(l:{child.sourceline}) in '{file}' have not main xml node='manifest'")
        for child in root:
            if type(child) == etree._Comment:
                debug.verbose(f"(l:{child.sourceline})     comment='{child.text}'")
                continue
            if child.tag == "remote":
                name = "origin"
                fetch = ""
                for attr in child.attrib:
                    if attr == "name":
                        name = child.attrib[attr]
                    elif attr == "fetch":
                        fetch = child.attrib[attr]
                        if len(fetch) >= 2 and fetch[:2] == "..":
                            # we have a relative island manifest ==> use local manifest origin to get the full origin
                            cmd = "git remote get-url origin"
                            debug.verbose(f"execute : {cmd}")
                            base_origin = multiprocess.run_command(cmd, cwd=env.get_island_path_manifest())
                            debug.verbose(f"base_origin={base_origin[1]}")
                            base_origin = base_origin[1]
                            while len(fetch) >= 2 and fetch[:2] == "..":
                                fetch = fetch[2:]
                                while len(fetch) >= 1 and (fetch[0] == "/" or fetch[0] == "\\"):
                                    fetch = fetch[1:]
                                offset_1 = base_origin.rfind("/")
                                offset_2 = base_origin.rfind(":")
                                if offset_1 > offset_2:
                                    base_origin = base_origin[:offset_1]
                                else:
                                    base_origin = base_origin[:offset_2]
                            debug.verbose(f"new base_origin={base_origin}")
                            debug.verbose(f"tmp fetch={fetch}")
                            if fetch != "":
                                fetch = f"{base_origin}/{fetch}"
                            else:
                                fetch = base_origin
                            debug.verbose(f"new fetch={fetch}")
                        while len(fetch) > 1 and (fetch[-1] == "\\" or fetch[-1] == "/"):
                            fetch = fetch[:-1]
                    else:
                        debug.error(
                            f"(l:{child.sourceline}) Parsing the manifest : unknown '{child.tag}'  attribute : '{attr}', available:[name,fetch]"
                        )
                debug.debug(f"(l:{child.sourceline})     find '{child.tag}' : name='{name}' fetch='{fetch}'")
                # parse the sub global mirror list
                mirror_list = []
                for child_2 in child:
                    if child_2.tag == "mirror":
                        # find a new mirror
                        mirror_name = ""
                        mirror_fetch = ""
                        for attr_2 in child_2.attrib:
                            if attr_2 == "name":
                                mirror_name = child_2.attrib[attr_2]
                            elif attr_2 == "fetch":
                                mirror_fetch = child_2.attrib[attr_2]
                                while len(mirror_fetch) > 1 and (mirror_fetch[-1] == "\\" or mirror_fetch[-1] == "/"):
                                    mirror_fetch = mirror_fetch[:-1]
                            else:
                                debug.error(
                                    f"(l:{child_2.sourceline}) Parsing the manifest : unknown '{child_2.tag}'  attribute : '{attr_2}', available:[name,fetch]"
                                )
                        debug.debug(f"mirror: '{mirror_name}' '{mirror_fetch}'")
                        if mirror_name == "":
                            debug.error(f"(l:{child_2.sourceline}) Missing mirror 'name'")
                        if mirror_fetch == "":
                            debug.error(f"(l:{child_2.sourceline}) Missing mirror 'fetch'")
                        mirror_list.append(
                            {
                                "name": mirror_name,
                                "fetch": mirror_fetch,
                            }
                        )
                    else:
                        debug.error(f"(l:{child_2.sourceline}) Parsing the manifest : unknown '{child_2.tag}', available:[mirror]")
                self.remotes.append({"name": name, "fetch": fetch, "mirror": mirror_list})
                continue

            if child.tag == "import":
                type_manifest = "vcs"
                name = ""
                for attr in child.attrib:
                    if attr == "type":
                        type_manifest = child.attrib[attr]
                        if type_manifest not in ["vcs"]:
                            debug.error(
                                f"(l:{child.sourceline}) Parsing the manifest: {child.tag} attribute '{attr}={type_manifest}' value available: [vcs]"
                            )
                    elif attr == "name":
                        name = child.attrib[attr]
                    else:
                        debug.error(f"(l:{child.sourceline}) Parsing the manifest : unknown '{child.tag}'  attribute : '{attr}', available:[name]")
                new_name_yaml = os.path.join(os.path.dirname(self.manifest_xml), name)
                if os.path.exists(new_name_yaml) is False:
                    debug.error(f"(l:{child.sourceline}) The file does not exist : '{new_name_yaml}'")
                self.imports.append({"name": name, "path": new_name_yaml, "type": type_manifest})
                continue
            if child.tag == "include":
                name = ""
                for attr in child.attrib:
                    if attr == "name":
                        name = child.attrib[attr]
                    else:
                        debug.error(f"(l:{child.sourceline}) Parsing the manifest : unknown '{child.tag}'  attribute : '{attr}', available:[name]")
                debug.debug(f"(l:{child.sourceline})     find '{child.tag}' : name='{name}'")
                # check if the file exist ...
                new_name_xml = os.path.join(os.path.dirname(self.manifest_xml), name)
                if os.path.exists(new_name_xml) is False:
                    debug.error(f"(l:{child.sourceline}) The file does not exist : '{new_name_xml}'")
                self.includes.append({"name": name, "path": new_name_xml, "manifest": None})
                continue
            if child.tag == "option":
                remote = "origin"
                deliver_master = "master"
                sync = False
                deliver_source = "dev"
                default_branch = "dev"
                deliver_mode = "fast_forward"

                for child_2 in child:
                    if child_2.tag == "branch-release":
                        deliver_master = child_2.text
                    elif child_2.tag == "branch-develop":
                        deliver_source = child_2.text
                    elif child_2.tag == "default-branch":
                        default_branch = child_2.text
                    elif child_2.tag == "default-remote":
                        remote = child_2.text
                    elif child_2.tag == "deliver-mode":
                        deliver_mode = child_2.text
                        if deliver_mode not in ["merge", "fast_forward"]:
                            debug.error(f"(l:{child.sourceline}) Parsing the manifest: option 'deliver-mode' value available: [merge,fast_forward]")
                    elif child_2.tag == "synchronize-submodule":
                        sync_tmp = child_2.text
                        if sync_tmp.lower() == "true" or sync_tmp == "1" or sync_tmp.lower() == "yes":
                            sync = True
                        elif sync_tmp.lower() == "false" or sync_tmp == "0" or sync_tmp.lower() == "no":
                            sync = False
                        else:
                            debug.error(
                                f"(l:{child.sourceline}) Parsing the manifest : unknown '{child.tag}/{child2.tag}', value:'{sync}' available:[true,1,yes,false,0,no]"
                            )
                    else:
                        debug.error(
                            f"(l:{child_2.sourceline}) Parsing the manifest : unknown '{child.tag}/{child_2.tag}', available:[branch-release,branch-develop,default-branch,default-remote,synchronize-submodule]"
                        )
                self.default = {
                    "remote": remote,
                    "revision": deliver_master,
                    "sync": sync,
                    "branch-develop": deliver_source,
                    "default-branch": default_branch,
                    "deliver-mode": deliver_mode,
                }
                self.deliver_master = deliver_master
                self.deliver_develop = deliver_source
                self.deliver_mode = deliver_mode
                debug.debug(f"(l:{child.sourceline})     find '{child.tag}':")
                debug.debug(f"    - default-branch:'{default_branch}':")
                debug.debug(f"    - default-remote:'{remote}':")
                debug.debug(f"    - synchronize-submodule:'{sync}':")
                debug.debug(f"    - branch-release:'{deliver_master}':")
                debug.debug(f"    - branch-develop:'{deliver_source}':")
                debug.debug(f"    - deliver-mode:'{deliver_mode}':")
                continue
            if child.tag == "project":
                name = ""
                path = ""
                tag_sha1 = None
                for attr in child.attrib:
                    if attr == "name":
                        name = child.attrib[attr]
                    elif attr == "path":
                        path = child.attrib[attr]
                    elif attr == "tag":
                        tag_sha1 = child.attrib[attr]
                    else:
                        debug.error(
                            f"(l:{child.sourceline}) Parsing the manifest: unknown '{child.tag}'  attribute : '{attr}', available:[name,tag,sync-s]"
                        )
                if name == "":
                    debug.error(
                        f"(l:{child.sourceline}) Parsing the manifest: '{child.tag}'  missing attribute: 'name' ==> specify the git to clone ..."
                    )
                self.projects.append(
                    {
                        "name": name,
                        "path": path,
                        "tag": tag_sha1,
                    }
                )
                debug.debug(f"(l:{child.sourceline})     find '{child.tag}' : name='{name}' path='{path}' tag='{str(tag_sha1)}'")
                continue
            if child.tag == "link":
                # not managed ==> future use
                source = ""
                destination = ""
                for attr in child.attrib:
                    if attr == "source":
                        source = child.attrib[attr]
                    elif attr == "destination":
                        destination = child.attrib[attr]
                    else:
                        debug.error(
                            f"(l:{child.sourceline}) Parsing the manifest: unknown '{child.tag}'  attribute : '{attr}', available:[source,destination]"
                        )
                if source == "":
                    debug.error(
                        f"(l:{child.sourceline}) Parsing the manifest: '{child.tag}'  missing attribute: 'source' ==> specify the git to clone."
                    )
                if destination == "":
                    debug.error(
                        f"(l:{child.sourceline}) Parsing the manifest: '{child.tag}'  missing attribute: 'destination' ==> specify the git to clone."
                    )
                self.links.append(
                    {
                        "source": source,
                        "destination": destination,
                    }
                )
                debug.debug(f"Add link: '{destination}' ==> '{source}'")
                continue
            debug.info(f"(l:{child.sourceline})     '{child.tag}' values={child.attrib}")
            debug.error(f"(l:{child.sourceline}) Parsing error unknown NODE : '{child.tag}' available:[remote,include,default,project,option,link]")
        # now we parse all sub repo:
        for elem in self.includes:
            elem["manifest"] = Manifest(elem["path"])
        for elem in self.imports:
            elem["manifest"] = ManifestVSC(elem["path"])

        # inside data    child.text

    def _create_path_with_elem(self, element):
        path = element["path"]
        if path == "":
            path = element["name"]
            if len(path) >= 4 and path[-4:] == ".git":
                path = path[:-4]
        return path

    def _check_double_path(self, list_path=[], space=""):
        debug.debug(f"{space}check path : '{self.manifest_xml}'")
        for elem in self.projects:
            path = self._create_path_with_elem(elem)
            debug.debug(f"{space}    check path:'{path}'")
            if path in list_path:
                debug.error(f"Check Manifest error : double use of the path '{path}'")
            list_path.append(path)
        for elem in self.includes:
            elem["manifest"]._check_double_path(list_path, space + "    ")

    def get_all_configs(self, default=None, upper_remotes=[]):
        out = []
        if default == None:
            if self.default != None:
                default = copy.deepcopy(self.default)
            else:
                default = copy.deepcopy(self.default_base)
        # debug.error(f" self.default={self.default}")
        # add all local project
        for elem in self.projects:
            debug.verbose(f"parse element {elem}")
            if env.need_process_with_filter(elem["name"]) is False:
                debug.info(f"Filter repository: {elem['name']}")
                continue
            conf = repo_config.RepoConfig()
            conf.name = elem["name"]
            conf.tag = elem["tag"]
            conf.path = self._create_path_with_elem(elem)

            # add default remote for the project (search in inherited element)
            for remote in self.remotes:
                debug.verbose(f"    Local Remote: {remote}")
                if remote["name"] == default["remote"]:
                    conf.remotes.append(remote)
            if len(conf.remotes) == 0:
                for remote in upper_remotes:
                    debug.verbose(f"    upper Remote: {remote}")
                    if remote["name"] == default["remote"]:
                        conf.remotes.append(remote)
            if len(conf.remotes) == 0:
                debug.error(
                    f"    No remote detected: {len(conf.remotes)} for {conf.name} with default remote name : {default['remote']} self remote: {self.remotes}"
                )

            # select default remote:
            conf.select_remote = None
            debug.debug(f"    remotes count: {len(conf.remotes)}")
            for remote in conf.remotes:
                debug.debug(f"    remote={remote}")
                debug.debug(f"    Check remote : {remote['name']} == {default['remote']}")
                debug.verbose(f"          remote={remote}")
                debug.verbose(f"          default={default}")
                if remote["name"] == default["remote"]:
                    conf.select_remote = copy.deepcopy(remote)
                    debug.debug(f"    copy select={conf.select_remote}")

                    # copy the submodule synchronization
                    conf.select_remote["sync"] = default["sync"]
                    break
            if conf.select_remote == None:
                debug.error(f"missing remote for project: {conf.name}")

            conf.branch = default["revision"]
            out.append(conf)
        # create a temporary variable to transmit the remote to includes
        upper_remotes_forward = copy.deepcopy(upper_remotes)
        for remote in self.remotes:
            upper_remotes_forward.append(remote)
        # add all include project
        for elem in self.includes:
            list_project = elem["manifest"].get_all_configs(default, upper_remotes_forward)
            for elem_proj in list_project:
                out.append(elem_proj)
        # add all import project
        for elem in self.imports:
            list_project = elem["manifest"].get_all_configs(default, upper_remotes_forward)
            for elem_proj in list_project:
                out.append(elem_proj)

        # -------------------------------------------------------------
        # -- add Volatile ...
        # -------------------------------------------------------------
        debug.verbose("include volatile config")
        # TODO: maybe find a better way to do this...
        conf_global = config.get_unique_config()
        for elem in conf_global.get_volatile():
            conf = repo_config.RepoConfig()
            base_volatile, repo_volatile = repo_config.split_repo(elem["git_address"])
            conf.name = repo_volatile
            conf.path = elem["path"]
            conf.branch = "master"
            conf.volatile = True
            conf.remotes = [{"name": "origin", "fetch": base_volatile, "mirror": []}]
            conf.select_remote = {
                "name": "origin",
                "fetch": base_volatile,
                "sync": False,
                "mirror": [],
            }
            out.append(conf)
        # -------------------------------------------------------------
        if False:
            debug.info("list of all repo:")
            for elem in out:
                debug.info(f"    '{elem.name}'")
                debug.info(f"        path: {elem.path}")
                debug.info(f"        remotes: {elem.remotes}")
                debug.info(f"        select_remote: {elem.select_remote}")
                debug.info(f"        branch: {elem.branch}")
        return out


def tag_manifest(manifest_xml_filename, all_tags):
    tree = etree.parse(manifest_xml_filename)
    debug.debug(f"manifest : '{manifest_xml_filename}'")
    root = tree.getroot()
    includes = []
    if root.tag != "manifest":
        debug.error("(l:{child.sourceline}) in '{file}' have not main xml node='manifest'")
        return False
    for child in root:
        if type(child) == etree._Comment:
            debug.verbose(f"(l:{child.sourceline})     comment='{child.text}'")
            continue
        if child.tag == "remote":
            continue
        if child.tag == "include":
            name = ""
            for attr in child.attrib:
                if attr == "name":
                    name = child.attrib[attr]
                else:
                    debug.error(f"(l:{child.sourceline}) Parsing the manifest : unknown '{child.tag}'  attribute : '{attr}', available:[name]")
            debug.debug(f"(l:{child.sourceline})     find '{child.tag}' : name='{name}'")
            # check if the file exist ...
            new_name_xml = os.path.join(os.path.dirname(manifest_xml_filename), name)
            if os.path.exists(new_name_xml) is False:
                debug.error(f"(l:{child.sourceline}) The file does not exist : '{new_name_xml}'")
            includes.append({"name": name, "path": new_name_xml, "manifest": None})
            continue
        if child.tag == "default":
            continue
        if child.tag == "project":
            name = ""
            path = ""
            tag_sha1 = None
            for attr in child.attrib:
                if attr == "name":
                    name = child.attrib[attr]
                elif attr == "path":
                    path = child.attrib[attr]
                elif attr == "tag":
                    tag_sha1 = child.attrib[attr]
                else:
                    debug.error(
                        f"(l:{child.sourceline}) Parsing the manifest: unknown '{child.tag}'  attribute : '{attr}', available:[name,tag,sync-s]"
                    )
            if name == "":
                debug.error(f"(l:{child.sourceline}) Parsing the manifest: '{child.tag}'  missing attribute: 'name' ==> specify the git to clone.")
            for elem_tag in all_tags:
                if elem_tag["name"] == name:
                    child.set("tag", elem_tag["tag"])
            continue
        if child.tag == "option":
            # not managed ==> future use
            continue
        if child.tag == "link":
            continue
        debug.info(f"(l:{child.sourceline})     '{child.tag}' values={child.attrib}")
        debug.error(f"(l:{child.sourceline}) Parsing error unknown NODE : '{child.tag}' available:[remote,include,default,project,option,link]")
    tree.write(manifest_xml_filename, pretty_print=True, xml_declaration=True, encoding="utf-8")
    # now we parse all sub repo:
    for elem in includes:
        tag_manifest(elem["path"], all_tags)


def tag_clear(manifest_xml_filename):
    tree = etree.parse(manifest_xml_filename)
    debug.debug(f"manifest : '{manifest_xml_filename}'")
    root = tree.getroot()
    includes = []
    if root.tag != "manifest":
        debug.error("(l:{child.sourceline}) in '{file}' have not main xml node='manifest'")
        return False
    for child in root:
        if type(child) == etree._Comment:
            debug.verbose("(l:{child.sourceline})     comment='{child.text}'")
            continue
        if child.tag == "remote":
            continue
        if child.tag == "include":
            name = ""
            for attr in child.attrib:
                if attr == "name":
                    name = child.attrib[attr]
                else:
                    debug.error(f"(l:{child.sourceline}) Parsing the manifest : unknown '{child.tag}'  attribute : '{attr}', available:[name]")
            debug.debug("(l:{child.sourceline})     find '{child.tag}' : name='{name}'")
            # check if the file exist ...
            new_name_xml = os.path.join(os.path.dirname(manifest_xml_filename), name)
            if os.path.exists(new_name_xml) is False:
                debug.error("(l:{child.sourceline}) The file does not exist : '{new_name_xml}'")
            includes.append({"name": name, "path": new_name_xml, "manifest": None})
            continue
        if child.tag == "default":
            continue
        if child.tag == "project":
            child.attrib.pop("tag", None)
            continue
        if child.tag == "option":
            continue
        if child.tag == "link":
            continue
        debug.info(f"(l:{child.sourceline})     '{child.tag}' values={child.attrib}")
        debug.error(f"(l:{child.sourceline}) Parsing error unknown NODE : '{child.tag}' available:[remote,include,default,project,option,link]")
    tree.write(manifest_xml_filename, pretty_print=True, xml_declaration=True, encoding="utf-8")
    # now we parse all sub repo:
    for elem in includes:
        tag_clear(elem["path"])



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Config main interface.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""
import copy
import json
import os
from typing import Optional

# Local import
from realog import debug

from . import (
    env,
    repo_config,
    tools,
)


env.get_island_path_config()


class Config:
    def __init__(self) -> None:
        self._repo = ""
        self._branch = "master"
        self._manifest_name = "default.xml"
        self._volatiles = []
        self._current_link = []
        self.load()

    def load(self) -> bool:
        if os.path.exists(env.get_island_path_config()) is False:
            return True
        self._volatiles = []
        self._current_link = []
        with open(env.get_island_path_config()) as json_file:
            data = json.load(json_file)
            if "repo" in data.keys():
                self._repo = data["repo"]
            if "branch" in data.keys():
                self._branch = data["branch"]
            if "manifest_name" in data.keys():
                self._manifest_name = data["manifest_name"]
            if "volatiles" in data.keys():
                for elem in data["volatiles"]:
                    if "git_address" in elem.keys() and "path" in elem.keys():
                        self.add_volatile(elem["git_address"], elem["path"])
            if "link" in data.keys():
                for elem in data["link"]:
                    if "source" in elem.keys() and "destination" in elem.keys():
                        self.add_link(elem["source"], elem["destination"])
            return True
        return False

    def store(self) -> bool:
        data = {}
        data["repo"] = self._repo
        data["branch"] = self._branch
        data["manifest_name"] = self._manifest_name
        data["volatiles"] = self._volatiles
        data["link"] = self._current_link
        with open(env.get_island_path_config(), "w") as outfile:
            json.dump(data, outfile, indent=4)
            return True
        return False

    def set_manifest(self, value):
        self._repo = value

    def get_manifest(self):
        return self._repo

    def set_branch(self, value):
        self._branch = value

    def get_branch(self):
        return self._branch

    def set_manifest_name(self, value):
        self._manifest_name = value

    def get_manifest_name(self):
        return self._manifest_name

    def add_volatile(self, git_adress, local_path):
        for elem in self._volatiles:
            if elem["path"] == local_path:
                debug.error(
                    "can not have multiple local repositoty on the same PATH",
                    crash=False,
                )
                return False
        self._volatiles.append({"git_address": git_adress, "path": local_path})
        return True

    def get_volatile(self):
        return copy.deepcopy(self._volatiles)

    def get_links(self):
        return self._current_link

    def add_link(self, source, destination) -> bool:
        for elem in self._current_link:
            if elem["destination"] == destination:
                debug.error(
                    "can not have multiple destination folder in link " + destination,
                    crash=False,
                )
                return False
        self._current_link.append({"source": source, "destination": destination})
        return True

    def remove_link(self, destination) -> None:
        for elem in self._current_link:
            if elem["destination"] == destination:
                del self._current_link[elem]
                return
        debug.warning("Request remove link that does not exist")

    def clear_links(self):
        self._current_link = []

    def get_manifest_config(self):
        conf = repo_config.RepoConfig()
        base_volatile, repo_volatile = repo_config.split_repo(self.get_manifest())
        conf.name = repo_volatile
        conf.path = os.path.join("." + env.get_system_base_name(), "manifest")  # env.get_island_path_manifest()
        conf.branch = "master"
        conf.volatile = False
        conf.remotes = [{"name": "origin", "fetch": base_volatile, "mirror": []}]
        conf.select_remote = {
            "name": "origin",
            "fetch": base_volatile,
            "sync": False,
            "mirror": [],
        }
        return conf


_unique_config: Optional[Config] = None


def get_unique_config() -> Config:
    global _unique_config
    if _unique_config is None:
        _unique_config = Config()
    return _unique_config



# -*- coding: utf-8 -*-
"""Action script for volatile-list.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os

from realog import debug

from island import (
    commands,
    config,
    env,
    manifest,
    multiprocess,
    tools,
)


#
# @brief Get the global description of the current action
# @return (string) the description string (fist line if reserved for the overview, all is for the specific display)
#
def help() -> str:
    return "List all the volatile repository"


#
# @brief Execute the action required.
#
# @return error value [0 .. 50] the <0 value is reserved system ==> else, what you want.
#         None : No error (return program out 0)
#         -5   : env.ret_manifest_is_not_existing      : Manifest does not exit
#         -10  : env.ret_action_is_not_existing        : ACTION is not existing
#         -11  : env.ret_action_executing_system_error : ACTION execution system error
#         -12  : env.ret_action_wrong_parameters       : ACTION Wrong parameters
#         -13  : env.ret_action_partial_done           : ACTION partially done
#
def execute(_arguments):
    for elem in _arguments:
        debug.error("Wrong argument: '" + elem.get_option_name() + "' '" + elem.get_arg() + "'")

    # check system is OK
    manifest.check_island_is_init()

    conf = config.get_unique_config()
    volatiles = conf.get_volatile()
    debug.info("List of all volatiles repository: ")
    for elem in volatiles:
        debug.info("\t" + elem["path"] + "\r\t\t\t\t" + elem["git_address"])

    return None



# -*- coding: utf-8 -*-
"""Action script for deliver.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os

from realog import debug
import status

from island import (
    commands,
    config,
    env,
    manifest,
    multiprocess,
    tools,
)


#
# @brief Get the global description of the current action
# @return (string) the description string (fist line if reserved for the overview, all is for the specific display)
#
def help():
    return "Deliver the current repository (develop & master MUST be up to date and you MUST be on master)"


#
# @brief Add argument to the specific action
# @param[in,out] my_args (death.Arguments) Argument manager
# @param[in] section Name of the currect action
#
def add_specific_arguments(_my_args, _section):
    _my_args.add("f", "from", haveParam=True, desc="source branche to deliver")
    _my_args.add("t", "to", haveParam=True, desc="desticantion branche of the deliver")


#
# @brief Execute the action required.
#
# @return error value [0 .. 50] the <0 value is reserved system ==> else, what you want.
#         None : No error (return program out 0)
#         -10 : ACTION is not existing
#         -11 : ACTION execution system error
#         -12 : ACTION Wrong parameters
#
def execute(_arguments):
    argument_from = None
    argument_to = None
    for elem in _arguments:
        if elem.get_option_name() == "from":
            debug.info("find source branch name: '" + elem.get_arg() + "'")
            argument_from = elem.get_arg()
        elif elem.get_option_name() == "to":
            debug.info("find destination branch name: '" + elem.get_arg() + "'")
            argument_to = elem.get_arg()
        else:
            debug.error("Wrong argument: '" + elem.get_option_name() + "' '" + elem.get_arg() + "'")

    # check system is OK
    manifest.check_island_is_init()

    configuration = config.get_unique_config()

    file_source_manifest = os.path.join(env.get_island_path_manifest(), configuration.get_manifest_name())
    if os.path.exists(file_source_manifest) is False:
        debug.error("Missing manifest file : '" + str(file_source_manifest) + "'")

    mani = manifest.Manifest(file_source_manifest)

    destination_branch = mani.deliver_master
    source_branch = mani.deliver_develop
    if argument_from != None:
        source_branch = argument_from
    if argument_to != None:
        destination_branch = argument_to

    all_project = mani.get_all_configs()
    debug.info("Check if all project are on master: " + str(len(all_project)) + " projects")
    id_element = 0
    deliver_available = True
    for elem in all_project:
        id_element += 1
        base_display = tools.get_list_base_display(id_element, len(all_project), elem)
        debug.verbose("deliver-ckeck: " + base_display)
        if (
            status.deliver_check(
                elem,
                "origin",  # TODO: argument_remote_name,
                id_element,
                base_display,
                source_branch,
                destination_branch,
            )
            is False
        ):
            deliver_available = False
    if deliver_available is False:
        debug.error("deliver-ckeck: Correct the warning to validate the Merge")
        return
    debug.info("deliver-ckeck: ==> All is OK")
    id_element = 0
    for elem in all_project:
        id_element += 1
        base_display = tools.get_list_base_display(id_element, len(all_project), elem)
        debug.info("deliver: ========================================================================")
        debug.info("deliver: == " + base_display)
        debug.info("deliver: ========================================================================")

        git_repo_path = os.path.join(env.get_island_root_path(), elem.path)
        # Check the validity of the version,
        (
            version_description,
            add_in_version_management,
        ) = status.get_current_version_repo(git_repo_path)
        if version_description == None:
            continue
        debug.info("deliver:     ==> version: " + str(version_description))

        # go to the dev branch
        select_branch = commands.get_current_branch(git_repo_path)

        # Checkout destination branch:
        commands.checkout(git_repo_path, destination_branch)

        # create new repo tag
        new_version_description = status.create_new_version_repo(
            git_repo_path,
            version_description,
            add_in_version_management,
            source_branch,
            destination_branch,
        )
        debug.info("new version: " + str(new_version_description))
        if new_version_description == None:
            continue
        # merge branch
        if mani.deliver_mode == "merge":
            merge_force = True
        else:
            merge_force = False
        commands.merge_branch_on_master(
            git_repo_path,
            source_branch,
            merge_force,
            branch_destination=destination_branch,
        )

        version_path_file = os.path.join(git_repo_path, "version.txt")
        # update version file:
        tools.file_write_data(version_path_file, tools.version_to_string(new_version_description) + "\n")
        commands.add_file(git_repo_path, version_path_file)
        commands.commit_all(
            git_repo_path,
            "[RELEASE] Release v" + tools.version_to_string(new_version_description),
        )
        commands.tag(git_repo_path, "v" + tools.version_to_string(new_version_description))
        commands.checkout(git_repo_path, source_branch)
        commands.reset_hard(git_repo_path, destination_branch)
        new_version_description.append("dev")
        tools.file_write_data(version_path_file, tools.version_to_string(new_version_description) + "\n")
        commands.add_file(git_repo_path, version_path_file)
        commands.commit_all(git_repo_path, status.default_update_message)
        commands.checkout(git_repo_path, destination_branch)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Type of host detection.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""
import platform

from realog import debug


if platform.system() == "Linux":
    OS = "Linux"
elif platform.system() == "Windows":
    OS = "Windows"
elif platform.system() == "Darwin":
    OS = "MacOs"
else:
    debug.error(f"Unknown the Host OS ... '{platform.system()}'")

debug.debug("host.OS = {OS}")



# -*- coding: utf-8 -*-
"""Action script for manifest-checkout.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os

from realog import debug
import status

from island import (
    commands,
    config,
    env,
    manifest,
    multiprocess,
    tools,
)


#
# @brief Get the global description of the current action
# @return (string) the description string (fist line if reserved for the overview, all is for the specific display)
#
def help():
    return "Manifest Ckeckout a specific branch of repository"


#
# @brief Add argument to the specific action
# @param[in,out] my_args (death.Arguments) Argument manager
# @param[in] section Name of the currect action
#
def add_specific_arguments(my_args, section):
    my_args.add("r", "remote", haveParam=True, desc="Name of the remote server")
    my_args.add_arg(
        "branch",
        optionnal=False,
        desc="Branch to checkout (if '__TAG__' ==> checkout specific repository tags)",
    )


#
# @brief Execute the action required.
#
# @return error value [0 .. 50] the <0 value is reserved system ==> else, what you want.
#         None : No error (return program out 0)
#         -10 : ACTION is not existing
#         -11 : ACTION execution system error
#         -12 : ACTION Wrong parameters
#
def execute(_arguments):
    argument_remote_name = ""
    branch_to_checkout = ""
    for elem in _arguments:
        if elem.get_option_name() == "remote":
            debug.info("find remote name: '" + elem.get_arg() + "'")
            argument_remote_name = elem.get_arg()
        elif elem.get_option_name() == "branch":
            branch_to_checkout = elem.get_arg()
        else:
            debug.error("Wrong argument: '" + elem.get_option_name() + "' '" + elem.get_arg() + "'")

    # check system is OK
    manifest.check_island_is_init()

    configuration = config.get_unique_config()

    elem = configuration.get_manifest_config()
    base_display = tools.get_list_base_display(0, 0, elem)
    if status.checkout_elem(elem, argument_remote_name, branch_to_checkout, base_display) is False:
        return env.ret_action_fail



# -*- coding: utf-8 -*-
"""Action script for manifest-sync.

@author Edouard DUPIN
@copyright 2012, Edouard DUPIN, all right reserved
@license MPL v2.0 (see license file)
"""

import os

from realog import debug

from island import (
    commands,
    config,
    env,
    manifest,
    multiprocess,
    tools,
)


#
# @brief Get the global description of the current action
# @return (string) the description string (fist line if reserved for the overview, all is for the specific display)
#
def help():
    return "Syncronize all the repository referenced"


#
# @brief Add argument to the specific action
# @param[in,out] my_args (death.Arguments) Argument manager
# @param[in] section Name of the currect action
#
def add_specific_arguments(my_args, section):
    pass


#
# @brief Execute the action required.
#
# @return error value [0 .. 50] the <0 value is reserved system ==> else, what you want.
#         None : No error (return program out 0)
#         -10 : ACTION is not existing
#         -11 : ACTION execution system error
#         -12 : ACTION Wrong parameters
#
def execute(_arguments):
    for elem in _arguments:
        debug.error("pull Wrong argument: '" + elem.get_option_name() + "' '" + elem.get_arg() + "'")

    # check system is OK
    manifest.check_island_is_init()

    configuration = config.get_unique_config()

    debug.info("update manifest : '" + str(env.get_island_path_manifest()) + "'")
    is_modify_manifest = commands.check_repository_is_modify(env.get_island_path_manifest())
    if is_modify_manifest is True:
        commands.fetch(env.get_island_path_manifest(), "origin")
    else:
        commands.pull(env.get_island_path_manifest(), "origin")
